#include "parser.h"
#include "printer.h"
#include "polynomial.h"
#include "division.h"
#include "lp.h"
#include "gfanapplication.h"
#include "polyhedralcone.h"
#include "polyhedralfan.h"
#include "symmetry.h"
#include "log.h"

#include "polymakefile.h"

class CheckRidgesApplication : public GFanApplication
{
  StringOption input1Option;
  StringOption input2Option;
  SimpleOption optionSymmetry;
public:
  bool includeInDefaultInstallation()
  {
    return false;
  }
  const char *helpText()
  {
    return "This program takes two pure fans F1 and F2 of the same dimension with the support of F1 contained in the support of F2. F2 must have polyhedral structure compatible with the common refinements of normal fans of a set of polynomials. These polynomial are specified on the standard input..\n";
  }
  CheckRidgesApplication():
    input1Option("-i1","Specify the name of the input file containing F1.","polymake.out"),
    input2Option("-i2","Specify the name of the input file containing F2.","polymake.out"),
    optionSymmetry("--symmetry",
		   "Reads in the cone stored with symmetry. The generators of the symmetry group are given on the input. IF SYMMETRY IS SPECIFIED THEN "
		   "THE CHECK IS NOT COMPLETE AND IT WILL ONLY BE CHECKED IF THAT GIVEN"
		   "TWO CONES THAT EXIST ELEMENTS IN THE TWO RESPECTIVE ORBITS"
		   "WHICH ARE CONNECTED BY A RIDGE PATH\n")
  {
    registerOptions();
  }    

  char *name()
  {
    return "_checkridges";
  }

  int main()
  {
    PolyhedralFan f1=PolyhedralFan::readFan(input1Option.getValue(),true,0,0,0,true);
    PolyhedralFan f2=PolyhedralFan::readFan(input2Option.getValue(),true,0,0,0,true);

    PolynomialSet g=FileParser(Stdin).parsePolynomialSetWithRing();

    int n=f1.getAmbientDimension();
    SymmetryGroup s(n);

    if(optionSymmetry.getValue())
      {
	IntegerVectorList generators=FileParser(Stdin).parseIntegerVectorList();
	s.computeClosure(generators);
      }


    log2 cerr<<"Changing the structure of f1" <<endl;
    
    for(PolynomialSet::const_iterator i=g.begin();i!=g.end();i++)
      {
	log2 cerr << "Computing normal fan of Newton polytope" <<endl;
	PolyhedralFan temp=PolyhedralFan::normalFanOfNewtonPolytope(*i);
	log2 cerr << "Refining" <<endl;
	f1=refinement(f1,temp);
	log2 cerr << "Removing lower dimensional" <<endl;
	f1.removeAllLowerDimensional();
	log2 cerr << "Cones in refinement: " << f1.size() << endl;
      }
    
    log2 cerr<<"Computing ridges of f1 with new fan structure"<<endl;

    PolyhedralFan ridges1=f1.facetComplex();

    log2 cerr << "Number of ridges in f1 representation " <<ridges1.size()<<endl;

    log2 cerr << "Here we could remove ridges until we have only one in each orbit."<<endl;

    log2 cerr << "Computing ridges of f2"<<endl;
    f2.removeAllLowerDimensional();
    PolyhedralFan ridges2=f2.facetComplex();

    log2 cerr << "Number of ridges in f2 representation " <<ridges2.size()<<endl;

    log2 cerr << "Here we could remove ridges until we have only one in each orbit."<<endl;

    log2 cerr << "Iterating through ridges of f2"<<endl;

    int I=0;
    for(PolyhedralFan::coneIterator i=ridges2.conesBegin();i!=ridges2.conesEnd();i++,I++)
      {
	log2 cerr << "Processing ridge "<<I<<endl;
	IntegerVector v=i->getUniquePoint();
	set<IntegerVector> pointsToCheck;
	for(PolyhedralFan::coneIterator j=ridges1.conesBegin();j!=ridges1.conesEnd();j++)
	  {
	    IntegerVector w=j->getUniquePoint();
	    for(SymmetryGroup::ElementContainer::const_iterator k=s.elements.begin();k!=s.elements.end();k++)
	      {
		IntegerVector u=SymmetryGroup::compose(*k,w);
		if(i->contains(u))
		  {
		    pointsToCheck.insert(w);//HERE WE ARE INSERTING AS f1 representatives
		    break;
		  }
	      }
	  }

	IntegerVectorList ptc;
	for(set<IntegerVector>::const_iterator j=pointsToCheck.begin();j!=pointsToCheck.end();j++)ptc.push_back(*j);

	AsciiPrinter P(Stdout);
	P<< "-----------------\n Need to compare ";
	P<< v;
	P<< " in F2 to the following vectors in F1 (these are in according to the original representatives of F1):\n";
	P << ptc;

	PolyhedralFan link2=f2.link(v,&s);
	link2.removeAllLowerDimensional();
	cout << "Number of rays in link of f2:" << link2.size() << endl;
	int nRays2=link2.size();
	for(IntegerVectorList::const_iterator j=ptc.begin();j!=ptc.end();j++)
	  {
	    PolyhedralFan link1=f1.link(*j,&s);
	    link1.removeAllLowerDimensional();
	    cout << "Number of rays in link of f1:" << link1.size() << endl;
	    if(nRays2!=link1.size())
	      {
		cerr << "THIS SHOULD NOT HAPPEN IF THE STRUCTURE OF f1 HAS BEEN CHANGED!" <<endl;
		if(link1.dimensionOfLinealitySpace()!=link1.getMaxDimension())
		  {
		    link1.printWithIndices(&P,FPF_default);
		    link2.printWithIndices(&P,FPF_default);

		    assert(nRays2==link1.size());
		  }
	      }
	  }
      }

    return 0;
  }
};

static CheckRidgesApplication theApplication;

