#include "polyhedralcone.h"

#include "lp.h"
#include "subspace.h"
#include "symmetry.h"
#include "polymakefile.h"
#include "wallideal.h"
#include <sstream>
#include "triangulation.h"
#include "linalg.h"
#include "log.h"

//--------------------------------
// PolyhedralCone
//--------------------------------

static bool compareIntegerLists(IntegerVectorList const &a, IntegerVectorList const &b)
{
  assert(a.size()==b.size());

  IntegerVectorList::const_iterator B=b.begin();
  for(IntegerVectorList::const_iterator A=a.begin();A!=a.end();A++)
    {
      if(LexicographicTermOrder()(*A,*B))return true;
      if(LexicographicTermOrder()(*B,*A))return false;
      B++;
    }
  return false;
}

bool operator<(PolyhedralCone const &a, PolyhedralCone const &b)
{
  assert(a.state>=3);
  assert(b.state>=3);

  if(a.equations.size()<b.equations.size())return true;
  if(a.equations.size()>b.equations.size())return false;

  if(a.n<b.n)return true;
  if(a.n>b.n)return false;

  if(a.halfSpaces.size()<b.halfSpaces.size())return true;
  if(a.halfSpaces.size()>b.halfSpaces.size())return false;

  if(compareIntegerLists(a.equations,b.equations))return true;
  if(compareIntegerLists(b.equations,a.equations))return false;

  if(compareIntegerLists(a.halfSpaces,b.halfSpaces))return true;
  if(compareIntegerLists(b.halfSpaces,a.halfSpaces))return false;

  return false;
}

bool operator!=(PolyhedralCone const &a, PolyhedralCone const &b)
{
  return (a<b)||(b<a);
}


void PolyhedralCone::ensureStateAsMinimum(int s)
{
  if((state<1) && (s==1))
    {
      //log0 cerr<<"Number of inequalities: "<<halfSpaces.size()<<" Number of equations: "<<equations.size()<<endl;
     
      {
	IntegerMatrix temp=rowsToIntegerMatrix(equations,n);
	FieldMatrix m=integerMatrixToFieldMatrix(temp,Q);
	m.reduce();
	m.removeZeroRows();

	IntegerVectorList newInequalities;
	for(IntegerVectorList::const_iterator i=halfSpaces.begin();i!=halfSpaces.end();i++)
	  {
	    FieldVector w=integerVectorToFieldVector(*i,Q);
	    w=m.canonicalize(w);
	    if(!w.isZero())
	      newInequalities.push_back(w.primitive());
	  }

	halfSpaces=newInequalities;
	equations=fieldMatrixToIntegerMatrixPrimitive(m).getRows();
      }

      //      log0 cerr<<"Number of inequalities: "<<halfSpaces.size()<<" Number of equations: "<<equations.size()<<endl;

      //      log0 fprintf(Stderr,"+\n");
      removeRedundantRows(&halfSpaces,&equations,false);
      //log0 cerr<<"Number of inequalities: "<<halfSpaces.size()<<" Number of equations: "<<equations.size()<<endl;
      //log0 fprintf(Stderr,"+\n");
    }
  if((state<2) && (s>=2))
    {
      //      halfSpaces.sort();
      //      AsciiPrinter(Stderr).printVectorList(halfSpaces);

       if(halfSpaces.size()>25)
	 //	if(0)
	 {
	  IntegerVectorList h1;
	  IntegerVectorList h2;
	  bool a=false;
	  for(IntegerVectorList::const_iterator i=halfSpaces.begin();i!=halfSpaces.end();i++)
	    {
	      if(a)
		h1.push_back(*i);
	      else
		h2.push_back(*i);
	      a=!a;
	    }
	  PolyhedralCone c1(h1,equations);
	  PolyhedralCone c2(h2,equations);
	  c1.ensureStateAsMinimum(2);
	  c2.ensureStateAsMinimum(2);
	  halfSpaces=c1.halfSpaces;
	  for(IntegerVectorList::const_iterator i=c2.halfSpaces.begin();i!=c2.halfSpaces.end();i++)
	    halfSpaces.push_back(*i);
	}
	

       //      fprintf(Stderr,"Number half spaces: %i, number of equations: %i\n",halfSpaces.size(),equations.size());
      if(equations.size())
	{
	  FieldMatrix equationSpace=integerMatrixToFieldMatrix(rowsToIntegerMatrix(equations,n),Q);
	  equationSpace.reduce();
	  IntegerVectorList halfSpaces2;
	  for(IntegerVectorList::const_iterator i=halfSpaces.begin();i!=halfSpaces.end();i++)
	    {
	      halfSpaces2.push_back(equationSpace.canonicalize(integerVectorToFieldVector(*i,Q)).primitive());
	    }
	  for(IntegerVectorList::const_iterator i=halfSpaces2.begin();i!=halfSpaces2.end();i++)
	    if((i->max()>1000) || (i->min()<-1000))goto fallBack;//more overflows caught in lp_cdd.
	  halfSpaces=fastNormals(halfSpaces2);
	  goto noFallBack;
	fallBack:
	  removeRedundantRows(&halfSpaces,&equations,true);
	noFallBack:;
	}
      else
	halfSpaces=fastNormals(halfSpaces);
      //   fprintf(Stderr,"done\n");
    }
  if((state<3) && (s>=3))
    {
      //      fprintf(Stderr,"Number half spaces: %i, number of equations: %i\n",halfSpaces.size(),equations.size());
      //      log1 fprintf(Stderr,"Computing subspace...\n");
      Subspace v(equations,n);
      
      equations=v.getRepresentation();
      //      log1 fprintf(Stderr,"...done computing subspace.\n");
      

      for(IntegerVectorList::iterator i=halfSpaces.begin();i!=halfSpaces.end();i++)
	{
	  *i=v.canonicalizeVector(*i);
	}
      halfSpaces.sort();
      //fprintf(Stderr,"done\n");
    }
  state=s;
}

void PolyhedralCone::canonicalize()
{
  ensureStateAsMinimum(3);
}


void PolyhedralCone::findFacets()
{
  ensureStateAsMinimum(2);
}


PolyhedralCone::PolyhedralCone(int ambientDimension):
  n(ambientDimension),
  state(1),
  multiplicity(-1)
{
}


PolyhedralCone::PolyhedralCone(IntegerVectorList const &halfSpaces_, IntegerVectorList const &equations_, int ambientDimension):
  halfSpaces(halfSpaces_),
  equations(equations_),
  state(0),
  multiplicity(-1)
{
  n=ambientDimension;
  if(n==-1)
    {
      if(!halfSpaces_.empty())
	n=halfSpaces_.begin()->size();
      else if(!equations_.empty())
	n=equations_.begin()->size();
      else
	{
	  assert(0);
	}
    }
  for(IntegerVectorList::const_iterator i=halfSpaces.begin();i!=halfSpaces.end();i++)
    {
      assert(i->size()==n);
    }
  for(IntegerVectorList::const_iterator i=equations.begin();i!=equations.end();i++)
    {
      /*AsciiPrinter P(Stderr);
      P.printString("FJDSKFJA\n");
      P.printVector(*i);
      */
      assert(i->size()==n);
    }
  ensureStateAsMinimum(1);
  //  computeAndReduceLinearitySpace();
}


IntegerVector PolyhedralCone::getRelativeInteriorPoint()const
{
  //  ensureStateAsMinimum(1);
  assert(state>=1);
  IntegerVector ret;
  IntegerVectorList g=equations;
  int numberOfEqualities=g.size();
  g.insert(g.end(),halfSpaces.begin(),halfSpaces.end());
  IntegerVector equalitySet(g.size());
  for(int i=0;i<numberOfEqualities;i++)equalitySet[i]=1;

  if(!g.empty())
    ret=relativeInteriorPoint(ambientDimension(),g,&equalitySet);
  else
    ret=IntegerVector(n);//cone is the full space, lp code would fail, since the dimension is unknown

  for(IntegerVectorList::const_iterator i=equations.begin();i!=equations.end();i++)
    {
      assert(dotLong(*i,ret)==0);
    }

  for(IntegerVectorList::const_iterator i=halfSpaces.begin();i!=halfSpaces.end();i++)
    {
      if(!(dotLong(*i,ret)>0))
	{
	  fprintf(Stderr,"PolyhedralCone::relativeInteriorPoint() : halfSpaces not reduced or mistake in cdd interface!!!\n");
	}
    }

  return ret;
}


IntegerVectorList PolyhedralCone::getHalfSpaces()const
{
  return halfSpaces;
}


const IntegerVectorList &PolyhedralCone::getEquations()const
{
  assert(state>=1);
  return equations;
}


int PolyhedralCone::ambientDimension()const
{
  return n;
}


int PolyhedralCone::codimension()const
{
  return ambientDimension()-dimension();
  //  return getEquations().size();
}


int PolyhedralCone::dimension()const
{
  assert(state>=1);
  //  ensureStateAsMinimum(1);
  return ambientDimension()-equations.size();
}


bool PolyhedralCone::isZero()const
{
  return dimension()==0;
}


PolyhedralCone intersection(const PolyhedralCone &a, const PolyhedralCone &b)
{
  assert(a.ambientDimension()==b.ambientDimension());
  IntegerVectorList halfSpaces=a.halfSpaces;
  halfSpaces.insert(halfSpaces.end(),b.halfSpaces.begin(),b.halfSpaces.end());
  IntegerVectorList equations=a.equations;
  equations.insert(equations.end(),b.equations.begin(),b.equations.end());
  return PolyhedralCone(halfSpaces,equations,a.ambientDimension());
}

PolyhedralCone product(const PolyhedralCone &a, const PolyhedralCone &b)
{
  IntegerVectorList equations2;
  IntegerVectorList inequalities2;

  int n1=a.n;
  int n2=b.n;

  for(IntegerVectorList::const_iterator i=a.equations.begin();i!=a.equations.end();i++)
    equations2.push_back(concatenation(*i,IntegerVector(n2)));
  for(IntegerVectorList::const_iterator i=b.equations.begin();i!=b.equations.end();i++)
    equations2.push_back(concatenation(IntegerVector(n1),*i));
  for(IntegerVectorList::const_iterator i=a.halfSpaces.begin();i!=a.halfSpaces.end();i++)
    inequalities2.push_back(concatenation(*i,IntegerVector(n2)));
  for(IntegerVectorList::const_iterator i=b.halfSpaces.begin();i!=b.halfSpaces.end();i++)
    inequalities2.push_back(concatenation(IntegerVector(n1),*i));

  PolyhedralCone ret(inequalities2,equations2,n1+n2);
  ret.setMultiplicity(a.getMultiplicity()*b.getMultiplicity());
  ret.setLinearForm(concatenation(a.getLinearForm(),b.getLinearForm()));

  ret.ensureStateAsMinimum(a.state);
  ret.ensureStateAsMinimum(b.state);

  return ret;
}

PolyhedralCone PolyhedralCone::positiveOrthant(int dimension)
{
  IntegerVectorList halfSpaces;

  for(int i=0;i<dimension;i++)halfSpaces.push_back(IntegerVector::standardVector(dimension,i));

  IntegerVectorList empty;
  return PolyhedralCone(halfSpaces,empty,dimension);
}


bool PolyhedralCone::isInStateMinimum(int s)const
{
  return state>=s;
}

int PolyhedralCone::getState()const
{
  return state;
}


void PolyhedralCone::print(class Printer *p, bool xml)const
{
  if(0)
    {
      p->printString("Printing PolyhedralCone");
      p->printNewLine();
      p->printString("Ambient dimension: ");
      p->printInteger(n);
      p->printNewLine();
      if(isInStateMinimum(1))
	{
	  p->printString("Dimension: ");
	  p->printInteger(dimension());
	  p->printNewLine();
	}
      p->printString("Linearity space:");
      //  p->printNewLine();
      p->printVectorList(equations);
      p->printString("Inequalities:");
      p->printVectorList(halfSpaces);
      p->printString("Relative interior point:\n");
      p->printVector(getRelativeInteriorPoint());
      p->printNewLine();
      p->printString("Done printing PolyhedralCone.");
      p->printNewLine();
    }
  else
    {
      PolymakeFile polymakeFile;
      polymakeFile.create("NONAME","PolyhedralCone","PolyhedralCone",xml);
      polymakeFile.writeCardinalProperty("AMBIENT_DIM",n);
      if(isInStateMinimum(1))
	{
	  polymakeFile.writeCardinalProperty("DIM",dimension());
	  //need to check that the following is done correctly
	  //       	  polymakeFile.writeCardinalProperty("LINEALITY_DIM",dimensionOfLinealitySpace());

	  polymakeFile.writeMatrixProperty("IMPLIED_EQUATIONS",rowsToIntegerMatrix(equations,n));
	}
      polymakeFile.writeCardinalProperty("LINEALITY_DIM",dimensionOfLinealitySpace());
      polymakeFile.writeMatrixProperty("LINEALITY_SPACE",rowsToIntegerMatrix(linealitySpace().dualCone().getEquations(),n));
      

      if(isInStateMinimum(2))
	polymakeFile.writeMatrixProperty("FACETS",rowsToIntegerMatrix(halfSpaces,n));
      else
	polymakeFile.writeMatrixProperty("INEQUALITIES",rowsToIntegerMatrix(halfSpaces,n));

      polymakeFile.writeCardinalVectorProperty("RELATIVE_INTERIOR_POINT",getRelativeInteriorPoint());


      stringstream s;
      polymakeFile.writeStream(s);
      string S=s.str();
      p->printString(S.c_str());
    }
}


static IntegerVector dehomogenize(IntegerVector const &v)
{
  assert(v.size()>0);

  IntegerVector ret(v.size()-1);

  for(int i=0;i<v.size()-1;i++)ret[i]=v[i];
  return ret;
}

static IntegerVectorList dehomogenize(IntegerVectorList const &l)
{
  IntegerVectorList ret;

  for(IntegerVectorList::const_iterator i=l.begin();i!=l.end();i++)
    {
      ret.push_back(dehomogenize(*i));
    }
  return ret;
}

PolyhedralCone PolyhedralCone::withLastCoordinateRemoved()const
{
  assert(n>0);

  return PolyhedralCone(dehomogenize(halfSpaces),dehomogenize(equations));
}

bool PolyhedralCone::containsPositiveVector()const
{
  PolyhedralCone temp=intersection(*this,PolyhedralCone::positiveOrthant(n));
  IntegerVector v=temp.getRelativeInteriorPoint();
  return v.isPositive();
}


int PolyhedralCone::dimensionOfLinealitySpace()const
{
  if(halfSpaces.empty())return dimension();
  IntegerVectorList a;
  PolyhedralCone temp(a,halfSpaces);
  temp=intersection(temp,*this);

  return temp.dimension();
}


bool PolyhedralCone::contains(IntegerVector const &v)const
{
  for(IntegerVectorList::const_iterator i=equations.begin();i!=equations.end();i++)
    {
      if(dotLong(*i,v)!=0)return false;
    }

  for(IntegerVectorList::const_iterator i=halfSpaces.begin();i!=halfSpaces.end();i++)
    {
      if(dotLong(*i,v)<0)return false;
    }

  return true;
}


bool PolyhedralCone::contains(PolyhedralCone const &c)const
{
  PolyhedralCone c2=intersection(*this,c);
  PolyhedralCone c3=c;
  c2.canonicalize();
  c3.canonicalize();
  return !(c2!=c3);
}


bool PolyhedralCone::containsRelatively(IntegerVector const &v)const
{
  assert(state>=1);
  for(IntegerVectorList::const_iterator i=equations.begin();i!=equations.end();i++)
    {
      if(dotLong(*i,v)!=0)return false;
    }

  for(IntegerVectorList::const_iterator i=halfSpaces.begin();i!=halfSpaces.end();i++)
    {
      if(dotLong(*i,v)<=0)return false;
    }

  return true;
}


PolyhedralCone PolyhedralCone::permuted(IntegerVector const &v)const
{
  PolyhedralCone ret(SymmetryGroup::permuteIntegerVectorList(halfSpaces,v),SymmetryGroup::permuteIntegerVectorList(equations,v),n);
  if(state>=1)ret.state=1;
  if(state>=2)ret.state=2;

  ret.ensureStateAsMinimum(state);
  return ret;
}

IntegerVector PolyhedralCone::getUniquePoint()const
{
  IntegerVectorList rays=extremeRays();
  IntegerVector ret(n);
  for(IntegerVectorList::const_iterator i=rays.begin();i!=rays.end();i++)
    ret+=*i;

  assert(containsRelatively(ret));//remove this check later
  return ret;
}

IntegerVectorList PolyhedralCone::extremeRays()const
{
  assert((dimension()==ambientDimension()) || (state>=3));
  /* This code actually works even for lower dimensional cones if they have been canonicalized. */

  
  //  AsciiPrinter(Stderr).printVectorList(halfSpaces);
  //  AsciiPrinter(Stderr).printVectorList(equations);

  IntegerVectorList ret;
  
  // log0 fprintf(Stderr,"calling double description (cddlib)\n");
  IntegerVectorList indices=extremeRaysInequalityIndices(halfSpaces);
  //  log0 fprintf(Stderr,"returning\n");
  // log0 fprintf(Stderr,"computing relative interior points\n");
  for(IntegerVectorList::const_iterator i=indices.begin();i!=indices.end();i++)
    {
      //  log0 AsciiPrinter(Stderr)<<*i;
      if(1)
	{
      /* At this point we know lineality space, implied equations and
	 also inequalities for the ray. To construct a vector on the
	 ray which is stable under (or indendent of) angle and
	 linarity preserving transformation we find the dimension 1
	 subspace orthorgonal to the implied equations and the
	 lineality space and pick a suitable primitive generator */
      
	  IntegerVector asVector(halfSpaces.size());
	  for(int j=0;j<i->size();j++)asVector[(*i)[j]]=1;
	  
	  IntegerVectorList equations=this->equations;
	  IntegerVectorList inequalities;
	  
	  IntegerVectorList::const_iterator a=halfSpaces.begin();
	  for(int j=0;j<asVector.size();j++,a++)
	    if(asVector[j])
	      equations.push_back(*a);

	  FieldMatrix linealitySpaceOrth=combineOnTop(integerMatrixToFieldMatrix(rowsToIntegerMatrix(this->equations,n),Q),integerMatrixToFieldMatrix(rowsToIntegerMatrix(this->halfSpaces,n),Q));
	  FieldMatrix temp=combineOnTop(linealitySpaceOrth.reduceAndComputeKernel(),integerMatrixToFieldMatrix(rowsToIntegerMatrix(equations,n),Q));
	  FieldMatrix temp2=temp.reduceAndComputeKernel();
	  
	  assert(temp2.getHeight()==1);
	  IntegerVector thePrimitiveVector=temp2[0].primitive();
	  if(!contains(thePrimitiveVector))thePrimitiveVector=-thePrimitiveVector;
	  ret.push_back(thePrimitiveVector);
	}
      else
	{
  /*      IntegerVectorList equations;
      for(int j=0;j<i->size();j++)
	{
	  IntegerVectorList::const_iterator a=halfSpaces.begin();
	  for(int k=0;k<(*i)[j];k++)
	    {
	      assert(a!=halfSpaces.end());
	      a++;
	    }
	  assert(a!=halfSpaces.end());
	  equations.push_back(*a);
	  }*/
  
	  IntegerVector asVector(halfSpaces.size());
	  for(int j=0;j<i->size();j++)asVector[(*i)[j]]=1;
	  
	  IntegerVectorList equations=this->equations;
	  IntegerVectorList inequalities;
	  
	  IntegerVectorList::const_iterator a=halfSpaces.begin();
	  for(int j=0;j<asVector.size();j++,a++)
	    if(asVector[j])
	      equations.push_back(*a);
	    else
	      inequalities.push_back(*a);
	  

	  //log0 fprintf(Stderr,"Equations %i, HalfSpaces: %i\n",equations.size(),inequalities.size());
	  IntegerVector u=PolyhedralCone(inequalities,equations).getRelativeInteriorPoint();
	  if(!u.isZero())
	    ret.push_back(u);
	  else
	    {
	      log2 fprintf(Stderr,"Remember to fix cdd double description interface\n");
	    }
	}
    }
  // log0 fprintf(Stderr,"done computing relative interior points\n");


  /*   //triangulation method. Keep this code.
  {
    IntegerMatrix temp=rowsToIntegerMatrix(halfSpaces);
 log0 fprintf(Stderr,"calling double description (triangulation)\n");
    IntegerVectorList ret2=Triangulation::normals(temp);
 log0 fprintf(Stderr,"returning\n");
 return ret2;

    AsciiPrinter(Stderr).printVectorList(halfSpaces);
    AsciiPrinter(Stderr).printVectorList(equations);
    fprintf(Stderr,"dim:%i\n",dimension());
    //ret.sort();
    AsciiPrinter(Stderr).printVectorList(ret);
    //  AsciiPrinter(Stderr).printVectorList(fastNormals(ret2));
    AsciiPrinter(Stderr).printVectorList(ret2);

    fprintf(stderr,"-----------------------\n");
  }
  */

  return ret;
}


bool PolyhedralCone::isSimplicial()const
{
  assert(state>=2);
  
  //  ensureStateAsMinimum(2);
  //  AsciiPrinter P(Stderr);
  //  print(&P);
  return codimension()+getHalfSpaces().size()+dimensionOfLinealitySpace()==n;
}


bool PolyhedralCone::checkDual(PolyhedralCone const &c)const
{
  assert(dimensionOfLinealitySpace()+c.dimension()==ambientDimension());

  for(IntegerVectorList::const_iterator i=halfSpaces.begin();i!=halfSpaces.end();i++)
    {
      assert(c.contains(*i));
    }
  for(IntegerVectorList::const_iterator i=equations.begin();i!=equations.end();i++)
    {
      assert(c.contains(*i));
    }
  return true;
}


PolyhedralCone PolyhedralCone::dualCone()const
{
  assert(state>=1);

  IntegerVectorList dualInequalities,dualEquations;

  dual(ambientDimension(),halfSpaces,equations,&dualInequalities,&dualEquations);

  PolyhedralCone ret(dualInequalities,dualEquations);

  ret.ensureStateAsMinimum(state);
  //  ret.canonicalize();


  assert(checkDual(ret));
  assert(ret.checkDual(*this));

  return ret;
}


PolyhedralCone PolyhedralCone::negated()const
{
  IntegerVectorList inequalities2;
  for(IntegerVectorList::const_iterator i=halfSpaces.begin();i!=halfSpaces.end();i++)inequalities2.push_back(-*i);
  PolyhedralCone ret(inequalities2,equations,n);
  ret.ensureStateAsMinimum(state);
  return ret;
}

PolyhedralCone PolyhedralCone::linealitySpace()const
{
  IntegerVectorList l1=getEquations();
  IntegerVectorList l2=getHalfSpaces();

  l1.splice(l1.begin(),l2);
  
  IntegerVectorList temp;
  PolyhedralCone ret(temp,l1,n);
  ret.ensureStateAsMinimum(state);
  return ret;
}


int PolyhedralCone::getMultiplicity()const
{
  return multiplicity;
}


void PolyhedralCone::setMultiplicity(int m)
{
  multiplicity=m;
}


IntegerVectorList PolyhedralCone::qoutientLatticeBasis()const
{
  assert(isInStateMinimum(1));// Implied equations must have been computed in order to know the span of the cone

  int a=equations.size();
  int b=halfSpaces.size();

  // Implementation below could be moved to nonLP part of code.
  
  // small vector space defined by a+b equations.... big by a equations.

  FieldMatrix M=combineLeftRight(combineLeftRight(
						  integerMatrixToFieldMatrix(rowsToIntegerMatrix(equations,n),Q).transposed(),
						  integerMatrixToFieldMatrix(rowsToIntegerMatrix(halfSpaces,n),Q).transposed()),
				 FieldMatrix::identity(Q,n));
  M.reduce(false,true);
  /*
    [A|B|I] is reduced to [A'|B'|C'] meaning [A'|B']=C'[A|B] and A'=C'A.

    [A'|B'] is in row echelon form, implying that the rows of C' corresponding to zero rows of [A'|B'] generate the lattice cokernel of [A|B] - that is the linealityspace intersected with Z^n.

    [A'] is in row echelon form, implying that the rows of C' corresponding to zero rows of [A'] generate the lattice cokernel of [A] - that is the span of the cone intersected with Z^n.

    It is clear that the second row set is a superset of the first. Their difference is a basis for the qoutient.
   */
  IntegerVectorList ret;

  for(int i=0;i<M.getHeight();i++)
    if(M[i].subvector(0,a).isZero()&&!M[i].subvector(a,a+b).isZero())ret.push_back(fieldVectorToIntegerVector(M[i].subvector(a+b,a+b+n)));

  return ret;
}


IntegerVector PolyhedralCone::semiGroupGeneratorOfRay()const
{
  IntegerVectorList temp=qoutientLatticeBasis();
  assert(temp.size()==1);
  for(IntegerVectorList::const_iterator i=halfSpaces.begin();i!=halfSpaces.end();i++)
    if(dotLong(temp.front(),*i)<0)
      {
	temp.front()=-temp.front();
	break;
      }
  return temp.front();
}


IntegerVector const &PolyhedralCone::getLinearForm()const
{
  return linearForm;
}


void PolyhedralCone::setLinearForm(IntegerVector const &linearForm_)
{
  linearForm=linearForm_;
}


PolyhedralCone PolyhedralCone::link(IntegerVector const &w)const
{
  assert(state>=3);
  IntegerVectorList inequalities2;
  for(IntegerVectorList::const_iterator j=halfSpaces.begin();j!=halfSpaces.end();j++)
    if(dotLong(w,*j)==0)inequalities2.push_back(*j);
  PolyhedralCone C(inequalities2,getEquations(),n);
  C.canonicalize();
  C.setLinearForm(getLinearForm());

  return C;  
}


PolyhedralCone PolyhedralCone::givenByRays(IntegerVectorList const &generators, IntegerVectorList const &linealitySpace, int n)
{
  //rewrite modulo lineality space
  IntegerVectorList newGenerators=generators;
  {
    Subspace l(linealitySpace,n);
    for(IntegerVectorList::const_iterator i=generators.begin();i!=generators.end();i++)
      newGenerators.push_back(l.canonicalizeVector(*i));
  }

  PolyhedralCone dual(newGenerators,linealitySpace,n);
  dual.findFacets();
  dual.canonicalize();
  IntegerVectorList inequalities=dual.extremeRays();
  
  IntegerVectorList span=generators;
  for(IntegerVectorList::const_iterator i=linealitySpace.begin();i!=linealitySpace.end();i++)span.push_back(*i);
  FieldMatrix m2Q=integerMatrixToFieldMatrix(rowsToIntegerMatrix(span,n),Q);
  IntegerVectorList equations=fieldMatrixToIntegerMatrixPrimitive(m2Q.reduceAndComputeKernel()).getRows();

  return PolyhedralCone(inequalities,equations,n);
}


FieldElement PolyhedralCone::volume()const
{
  AsciiPrinter P(Stderr);

  PolyhedralCone Ctemp=intersection(*this,this->linealitySpace().dualCone());

  Ctemp.canonicalize();
  cerr<<"testestests";
  IntegerVectorList eq2=rowsToIntegerMatrix(Ctemp.equations,n).transposed().getRows();
  eq2.push_front(IntegerVector(Ctemp.equations.size()));
  eq2=rowsToIntegerMatrix(eq2).transposed().getRows();
  cerr<<"testestests2";

  IntegerVectorList in2=rowsToIntegerMatrix(Ctemp.halfSpaces,n).transposed().getRows();
  in2.push_front(IntegerVector(Ctemp.halfSpaces.size()));
  in2=rowsToIntegerMatrix(in2).transposed().getRows();

  for(int i=0;i<n;i++)
    {
      in2.push_back(IntegerVector::standardVector(n+1,i+1)+IntegerVector::standardVector(n+1,0));
      //in2.push_back(-IntegerVector::standardVector(n+1,i+1)+IntegerVector::standardVector(n+1,0));
    }

  PolyhedralCone lifted(in2,eq2,n+1);

  lifted.canonicalize();
  IntegerMatrix A=rowsToIntegerMatrix(lifted.extremeRays()).transposed();//points are columns

  cerr << "height " << A.getHeight() << " width" <<A.getWidth()<<endl;

  P<<A.transposed().getRows();

  FieldMatrix A2(Q,A.getHeight()-1,A.getWidth());
  for(int i=0;i<A.getHeight()-1;i++)
    {
      A2[i]=integerVectorToFieldVector(A[i+1],Q)/integerVectorToFieldVector(A[0],Q);
    }
  A2=A2.transposed();//Now points are rows

  P<<"Triangulating\n";
  list<Triangulation::Cone> T=Triangulation::triangulate(A.transposed(),true);//revlex
  P<<"Done triangulating\n";

  FieldElement ret(Q);

  for(list<Triangulation::Cone>::const_iterator i=T.begin();i!=T.end();i++)
    {
      FieldMatrix S=A2.submatrixRows(*i);

      FieldMatrix S1(Q,S.getHeight()-1,S.getWidth());
      for(int j=0;j<S1.getHeight();j++)
	S1[j]=S[j+1]-S[0];

      FieldMatrix S2=S1*(S1.transposed());
      FieldElement square=S2.reduceAndComputeDeterminant();

      //      ret+=square.squareroot();
    }

  return ret;
}
